/***************************************************************************
 *            burn-caps.c
 *
 *  mar avr 18 20:58:42 2006
 *  Copyright  2006  Rouquier Philippe
 *  bonfire-app@wanadoo.fr
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>

#include <nautilus-burn-drive.h>

#include "burn-basics.h"
#include "burn.h"
#include "burn-caps.h"
 
static void bonfire_burn_caps_class_init (BonfireBurnCapsClass *klass);
static void bonfire_burn_caps_init (BonfireBurnCaps *sp);
static void bonfire_burn_caps_finalize (GObject *object);

struct BonfireBurnCapsPrivate {

};

typedef enum {
	LAST_SIGNAL
} BonfireBurnCapsSignalType;

//static guint bonfire_burn_caps_signals [LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

static BonfireBurnCaps *default_caps = NULL;

GType
bonfire_burn_caps_get_type ()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (BonfireBurnCapsClass),
			NULL,
			NULL,
			(GClassInitFunc)bonfire_burn_caps_class_init,
			NULL,
			NULL,
			sizeof (BonfireBurnCaps),
			0,
			(GInstanceInitFunc)bonfire_burn_caps_init,
		};

		type = g_type_register_static (G_TYPE_OBJECT,
					       "BonfireBurnCaps",
					       &our_info,
					       0);
	}

	return type;
}

static void
bonfire_burn_caps_class_init (BonfireBurnCapsClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);
	object_class->finalize = bonfire_burn_caps_finalize;
}

static void
bonfire_burn_caps_init (BonfireBurnCaps *obj)
{
	obj->priv = g_new0 (BonfireBurnCapsPrivate, 1);
	/* Initialize private members, etc. */
}

static void
bonfire_burn_caps_finalize (GObject *object)
{
	BonfireBurnCaps *cobj;

	cobj = BONFIRE_BURNCAPS(object);
	default_caps = NULL;
		
	g_free (cobj->priv);
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

BonfireBurnCaps *
bonfire_burn_caps_get_default ()
{
	if (!default_caps) 
		default_caps = BONFIRE_BURNCAPS (g_object_new (BONFIRE_TYPE_BURNCAPS, NULL));
	else
		g_object_ref (default_caps);

	return default_caps;
}

BonfireBurnResult
bonfire_burn_caps_get_default_flags (BonfireBurnCaps *caps,
				     BonfireTrackSourceType track_type,
				     NautilusBurnDriveType drive_type,
				     NautilusBurnMediaType media_type,
				     BonfireBurnFlag *flags)
{
	BonfireBurnFlag default_flags = BONFIRE_BURN_FLAG_CHECK_SIZE |
					  BONFIRE_BURN_FLAG_NOGRACE |
					  BONFIRE_BURN_FLAG_EJECT;

	if (drive_type != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
		default_flags |= BONFIRE_BURN_FLAG_BLANK_BEFORE_WRITE;
		default_flags |= BONFIRE_BURN_FLAG_BURNPROOF;

		if (track_type == BONFIRE_TRACK_SOURCE_GRAFTS
		||  track_type == BONFIRE_TRACK_SOURCE_DATA
		||  track_type == BONFIRE_TRACK_SOURCE_IMAGER)
			default_flags |= BONFIRE_BURN_FLAG_ON_THE_FLY;
	
		if (track_type == BONFIRE_TRACK_SOURCE_DISC
		&&  media_type <= NAUTILUS_BURN_MEDIA_TYPE_CDRW)
			default_flags |= BONFIRE_BURN_FLAG_ON_THE_FLY;
	}
	else
		default_flags |= BONFIRE_BURN_FLAG_DONT_CLEAN_OUTPUT;

	if (flags)
		*flags = default_flags;

	return BONFIRE_BURN_OK;
}

BonfireBurnResult
bonfire_burn_caps_get_supported_flags (BonfireBurnCaps *caps,
				       BonfireTrackSourceType track_type,
				       NautilusBurnDriveType drive_type,
				       NautilusBurnMediaType media_type,
				       BonfireBurnFlag *flags)
{
	BonfireBurnFlag supported_flags = BONFIRE_BURN_FLAG_NOGRACE |
					    BONFIRE_BURN_FLAG_EJECT;

	if (drive_type != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
		supported_flags |= BONFIRE_BURN_FLAG_BLANK_BEFORE_WRITE;
		supported_flags |= BONFIRE_BURN_FLAG_BURNPROOF;
		supported_flags |= BONFIRE_BURN_FLAG_DUMMY;

		if (track_type == BONFIRE_TRACK_SOURCE_GRAFTS
		||  track_type == BONFIRE_TRACK_SOURCE_DATA
		||  track_type == BONFIRE_TRACK_SOURCE_IMAGER
		||  track_type == BONFIRE_TRACK_SOURCE_DISC
		||  track_type == BONFIRE_TRACK_SOURCE_SONG)
			supported_flags |= BONFIRE_BURN_FLAG_ON_THE_FLY;
	}

	if (flags)
		*flags = supported_flags;

	return BONFIRE_BURN_OK;
}
