/***************************************************************************
 *            disc-monitor.h
 *
 *  lun jui 11 11:31:03 2005
 *  Copyright  2005  Rouquier Philippe
 *  bonfire-app@wanadoo.fr
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef DISC_MONITOR_H
#define DISC_MONITOR_H

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS
#define BONFIRE_TYPE_DISC_MONITOR         (bonfire_disc_monitor_get_type ())
#define BONFIRE_DISC_MONITOR(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), BONFIRE_TYPE_DISC_MONITOR, BonfireDiscMonitor))
#define BONFIRE_DISC_MONITOR_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), BONFIRE_TYPE_DISC_MONITOR, BonfireDiscMonitorClass))
#define BONFIRE_IS_DISC_MONITOR(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), BONFIRE_TYPE_DISC_MONITOR))
#define BONFIRE_IS_DISC_MONITOR_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), BONFIRE_TYPE_DISC_MONITOR))
#define BONFIRE_DISC_MONITOR_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), BONFIRE_TYPE_DISC_MONITOR, BonfireDiscMonitorClass))

typedef enum {
	BONFIRE_DISC_TYPE_NONE = 0,
	BONFIRE_DISC_TYPE_BLANK = 1,
	BONFIRE_DISC_TYPE_REWRITABLE = 1 << 1,
	BONFIRE_DISC_TYPE_AUDIO = 1 << 2,
	BONFIRE_DISC_TYPE_APPENDABLE = 1 << 3,
	BONFIRE_DISC_TYPE_MOUNTED = 1 << 4
} BonfireDiscType;

struct _BonfireDisc {
	char *udi;
	char *recorder;
	guint64 size;
	BonfireDiscType type;
};
typedef struct _BonfireDisc BonfireDisc;

typedef struct BonfireDiscMonitorPrivate BonfireDiscMonitorPrivate;

typedef struct {
	GObject parent;
	BonfireDiscMonitorPrivate *priv;
} BonfireDiscMonitor;

typedef struct {
	GObjectClass parent_class;

	void (*disc_added) (BonfireDiscMonitor * monitor,
			    const BonfireDisc * disc);
	void (*disc_removed) (BonfireDiscMonitor * monitor,
			      const BonfireDisc * disc);
} BonfireDiscMonitorClass;

GType bonfire_disc_monitor_get_type ();
BonfireDiscMonitor *bonfire_disc_monitor_get_default (void);

void bonfire_disc_free (BonfireDisc * disc);

GSList *bonfire_disc_monitor_lookup_disc (BonfireDiscMonitor * monitor, BonfireDiscType type);	/* at least one criterium */
GSList *bonfire_disc_monitor_lookup_disc_by_type (BonfireDiscMonitor * monitor, BonfireDiscType type);	/* must have all the criteria (but can have others) */
GSList *bonfire_disc_monitor_lookup_disc_by_type_strict (BonfireDiscMonitor * monitor, BonfireDiscType type);	/* must exactly match the criteria */

#endif				/* DISC_MONITOR_H */
