/***************************************************************************
*            file-src.c
*
*  jeu jun  2 18:51:16 2005
*  Copyright  2005  Philippe Rouquier
*  <bonfire-app@wanadoo.fr>
****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef BUILD_GDL

#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <glib/gi18n-lib.h>

#include <gtk/gtkvpaned.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkfilechooser.h>
#include <gtk/gtkfilechooserwidget.h>
#include <gtk/gtkfilefilter.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkaction.h>
#include <gtk/gtktoggleaction.h>

#include <gnome.h>

#include <gconf/gconf-client.h>

#include <gdl/gdl-dock.h>
#include <gdl/gdl-dock-item.h>
#include <gdl/gdl-dock-layout.h>
#include <gdl/gdl-dock-bar.h>
#include <gdl/gdl-dock-placeholder.h>

#include "file-src-gdl.h"

#ifdef BUILD_SEARCH
#include "search.h"
#endif

#ifdef BUILD_PLAYLIST
#include "play-list.h"
#endif

#ifdef BUILD_PREVIEW
#include "player.h"
#endif

typedef enum {
	BONFIRE_FILE_SOURCE_NO_PAGE,
	BONFIRE_FILE_SOURCE_CHOOSER_PAGE,

#ifdef BUILD_SEARCH
	BONFIRE_FILE_SOURCE_SEARCH_PAGE,
#endif

#ifdef BUILD_PLAYLIST
	BONFIRE_FILE_SOURCE_PLAYLIST_PAGE,
#endif
} BonfireFileSourcePage;


static void bonfire_file_source_class_init (BonfireFileSourceClass *
					    klass);
static void bonfire_file_source_init (BonfireFileSource * sp);
static void bonfire_file_source_finalize (GObject * object);
static void bonfire_file_source_destroy (GtkObject * gtkobject);

static void bonfire_file_source_chooser_selection_changed_cb (GtkFileChooser * chooser,
							      BonfireFileSource *source);
static void bonfire_file_source_chooser_activated_cb (GtkFileChooser *chooser,
						      BonfireFileSource *source);

#ifdef BUILD_SEARCH
static void bonfire_file_source_search_activated_cb (BonfireSearch *search,
						     BonfireFileSource *source);
static void bonfire_file_source_search_selection_changed_cb (BonfireSearch *search,
							     BonfireFileSource *source);
#endif

#ifdef BUILD_PLAYLIST
static void bonfire_file_source_playlist_activated_cb (BonfirePlaylist *playlist,
						       BonfireFileSource *source);
static void bonfire_file_source_playlist_selection_changed_cb (BonfirePlaylist *playlist,
							       BonfireFileSource *source);
#endif

static void bonfire_file_source_layout_changed_cb (GObject * object,
						   GParamSpec * pspec,
						   BonfireFileSource *source);
static void bonfire_file_source_focus_changed (BonfireFileSource * source,
					       BonfireFileSourcePage page);

struct BonfireFileSourcePrivate {
	GtkWidget *chooser;

#ifdef BUILD_SEARCH
	GtkWidget *search;
#endif

#ifdef BUILD_PLAYLIST
	GtkWidget *playlist;
#endif 

#ifdef BUILD_PREVIEW
	GtkWidget *viewer;
#endif

	BonfireFileSourcePage focus;
	GdlDockLayout *layout;
	GtkActionGroup *action_group;
};


/* MENUS */
static void bonfire_file_source_display_toggled_cb (GtkToggleAction *action,
						    BonfireFileSource *source);

static GtkActionEntry entries[] = {
	{"ViewMenu", NULL, N_("View")},
};

static GtkToggleActionEntry toggle_entries[] = {
	{"Chooser", NULL, N_("File chooser"), "F8", N_("Display file chooser"),
	 G_CALLBACK (bonfire_file_source_display_toggled_cb), FALSE},

#ifdef BUILD_PREVIEW
	{"Viewer", NULL, N_("Viewer"), "F7", N_("Display file viewer"),
	 G_CALLBACK (bonfire_file_source_display_toggled_cb), FALSE},
#endif

#ifdef BUILD_SEARCH
	{"Search", NULL, N_("Search"), "F9", N_("Display search"),
	 G_CALLBACK (bonfire_file_source_display_toggled_cb), FALSE},
#endif

#ifdef BUILD_PLAYLIST
	{"Playlist", NULL, N_("Playlist"), "F10", N_("Display playlist"),
	 G_CALLBACK (bonfire_file_source_display_toggled_cb), FALSE},
#endif
};

static const char *description = {
	"<ui>"
	    "<menubar name='menubar' >"
		"<menu action='ViewMenu'>"
			"<placeholder name='ViewPlaceholder'/>"
#ifdef BUILD_PREVIEW
			"<menuitem action='Viewer'/>"
#endif

			"<menuitem action='Chooser'/>"

#ifdef BUILD_SEARCH
			"<menuitem action='Search'/>"
#endif

#ifdef BUILD_PLAYLIST
			"<menuitem action='Playlist'/>"
#endif

			"<separator/>"
		"</menu>"
	    "</menubar>"
	"</ui>"
};


typedef enum {
	SELECTION_CHANGED_SIGNAL,
	ACTIVATED_SIGNAL,
	LAST_SIGNAL
} BonfireFileSourceSignalType;


static guint bonfire_file_source_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

GType
bonfire_file_source_get_type ()
{
	static GType type = 0;

	if (type == 0) {
		static const GTypeInfo our_info = {
			sizeof (BonfireFileSourceClass),
			NULL,
			NULL,
			(GClassInitFunc) bonfire_file_source_class_init,
			NULL,
			NULL,
			sizeof (BonfireFileSource),
			0,
			(GInstanceInitFunc) bonfire_file_source_init,
		};

		type = g_type_register_static (GDL_TYPE_DOCK,
					       "BonfireFileSource",
					       &our_info, 0);
	}

	return type;
}

static void
bonfire_file_source_class_init (BonfireFileSourceClass * klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);
	object_class->finalize = bonfire_file_source_finalize;
	gtkobject_class->destroy = bonfire_file_source_destroy;

	/* signals */
	bonfire_file_source_signals[ACTIVATED_SIGNAL] =
	    g_signal_new ("activated", G_OBJECT_CLASS_TYPE (object_class),
			  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			  G_STRUCT_OFFSET (BonfireFileSourceClass,
					   activated), NULL, NULL,
			  g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

	bonfire_file_source_signals[SELECTION_CHANGED_SIGNAL] =
	    g_signal_new ("selection_changed",
			  G_OBJECT_CLASS_TYPE (object_class),
			  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			  G_STRUCT_OFFSET (BonfireFileSourceClass,
					   selection_changed), NULL, NULL,
			  g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE,
			  1, G_TYPE_BOOLEAN);
}

static void
bonfire_file_source_init (BonfireFileSource * obj)
{
	GtkFileFilter *filter;

	obj->priv = g_new0 (BonfireFileSourcePrivate, 1);

	obj->priv->action_group =
	    gtk_action_group_new ("FileSourceActions");
	gtk_action_group_set_translation_domain (obj->priv->action_group,
						 NULL);
	gtk_action_group_add_actions (obj->priv->action_group, entries,
				      G_N_ELEMENTS (entries), obj);
	gtk_action_group_add_toggle_actions (obj->priv->action_group,
					     toggle_entries,
					     G_N_ELEMENTS (toggle_entries),
					     obj);
	gtk_action_group_set_sensitive (obj->priv->action_group, FALSE);

	/* CHOOSER */
	obj->priv->chooser =
	    gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_widget_show_all (obj->priv->chooser);

	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER
					 (obj->priv->chooser), TRUE);
	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER
					      (obj->priv->chooser), TRUE);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("All files"));
	gtk_file_filter_add_pattern (filter, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Audio files only"));
	gtk_file_filter_add_mime_type (filter, "audio/*");
	gtk_file_filter_add_mime_type (filter, "application/ogg");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Movies only"));
	gtk_file_filter_add_mime_type (filter, "video/*");
	gtk_file_filter_add_mime_type (filter, "application/ogg");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Image files only"));
	gtk_file_filter_add_mime_type (filter, "image/*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser),
				     filter);

	g_signal_connect (G_OBJECT (obj->priv->chooser), "file_activated",
			  G_CALLBACK
			  (bonfire_file_source_chooser_activated_cb), obj);
	g_signal_connect (G_OBJECT (obj->priv->chooser),
			  "selection_changed",
			  G_CALLBACK
			  (bonfire_file_source_chooser_selection_changed_cb),
			  obj);

#ifdef BUILD_PREVIEW
	obj->priv->viewer = bonfire_player_new ();
	gtk_widget_show_all (obj->priv->viewer);
#endif

#ifdef BUILD_SEARCH
	obj->priv->search = bonfire_search_new ();
	gtk_widget_show_all (obj->priv->search);

	g_signal_connect (G_OBJECT (obj->priv->search), "activated",
			  G_CALLBACK
			  (bonfire_file_source_search_activated_cb), obj);
	g_signal_connect (G_OBJECT (obj->priv->search),
			  "selection_changed",
			  G_CALLBACK
			  (bonfire_file_source_search_selection_changed_cb),
			  obj);
#endif

#ifdef BUILD_PLAYLIST
	obj->priv->playlist = bonfire_playlist_new ();
	gtk_widget_show_all (obj->priv->playlist);

	g_signal_connect (G_OBJECT (obj->priv->playlist), "activated",
			  G_CALLBACK (bonfire_file_source_playlist_activated_cb),
			  obj);
	g_signal_connect (G_OBJECT (obj->priv->playlist),
			  "selection_changed",
			  G_CALLBACK (bonfire_file_source_playlist_selection_changed_cb),
			  obj);
#endif

	/* NOTE: GDL object needs to be initialized first,
	 * so we cannot work on the layout from here */
}

static void
bonfire_file_source_destroy (GtkObject * gtkobject)
{
	BonfireFileSource *obj;

	obj = BONFIRE_FILE_SOURCE (gtkobject);

	if (obj->priv->layout) {
		char *path;

		gdl_dock_layout_save_layout (obj->priv->layout, "MainWin");
		path = gnome_util_home_file ("bonfire.layout");
		gdl_dock_layout_save_to_file (obj->priv->layout, path);
		g_free (path);

		g_object_unref (obj->priv->layout);
		obj->priv->layout = NULL;
	}

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		GTK_OBJECT_CLASS (parent_class)->destroy (gtkobject);
}

static void
bonfire_file_source_finalize (GObject * object)
{
	BonfireFileSource *cobj;

	cobj = BONFIRE_FILE_SOURCE (object);

	g_free (cobj->priv);
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

GtkWidget *
bonfire_file_source_new ()
{
	BonfireFileSource *obj;
	GtkWidget *chooser_item, *placeholder;

	obj =
	    BONFIRE_FILE_SOURCE (g_object_new
				 (BONFIRE_TYPE_FILE_SOURCE, NULL));

	/* Placeholders */
	placeholder =
	    gdl_dock_placeholder_new ("ph_top", GDL_DOCK_OBJECT (obj),
				      GDL_DOCK_TOP, FALSE);
	gtk_widget_show (placeholder);
	placeholder =
	    gdl_dock_placeholder_new ("ph_bot", GDL_DOCK_OBJECT (obj),
				      GDL_DOCK_BOTTOM, FALSE);
	gtk_widget_show (placeholder);
	placeholder =
	    gdl_dock_placeholder_new ("ph_left", GDL_DOCK_OBJECT (obj),
				      GDL_DOCK_LEFT, FALSE);
	gtk_widget_show (placeholder);
	placeholder =
	    gdl_dock_placeholder_new ("ph_right", GDL_DOCK_OBJECT (obj),
				      GDL_DOCK_RIGHT, FALSE);
	gtk_widget_show (placeholder);
	placeholder =
	    gdl_dock_placeholder_new ("ph_center", GDL_DOCK_OBJECT (obj),
				      GDL_DOCK_CENTER, FALSE);
	gtk_widget_show (placeholder);

#ifdef BUILD_PREVIEW
	GtkWidget *viewer_item =
	    gdl_dock_item_new_with_stock ("Viewer", _("File previewer"),
					  GTK_STOCK_FILE,
					  GDL_DOCK_ITEM_BEH_NEVER_VERTICAL
					  |
					  GDL_DOCK_ITEM_BEH_CANT_ICONIFY);
	gtk_container_add (GTK_CONTAINER (viewer_item), obj->priv->viewer);
	g_object_set (G_OBJECT (viewer_item), "resize", FALSE, NULL);
	gdl_dock_object_bind (GDL_DOCK_OBJECT (viewer_item),
			      G_OBJECT (GDL_DOCK_OBJECT_GET_MASTER (obj)));
#endif

	chooser_item =
	    gdl_dock_item_new_with_stock ("Chooser", _("File chooser"),
					  GTK_STOCK_DIRECTORY,
					  GDL_DOCK_ITEM_BEH_NEVER_HORIZONTAL
					  |
					  GDL_DOCK_ITEM_BEH_CANT_ICONIFY);
	gtk_container_add (GTK_CONTAINER (chooser_item), obj->priv->chooser);
	gdl_dock_object_bind (GDL_DOCK_OBJECT (chooser_item),
			      G_OBJECT (GDL_DOCK_OBJECT_GET_MASTER (obj)));

#ifdef BUILD_SEARCH
	GtkWidget *search_item =
	    gdl_dock_item_new_with_stock ("Search", _("File search"),
					  GTK_STOCK_FIND,
					  GDL_DOCK_ITEM_BEH_NEVER_HORIZONTAL
					  |
					  GDL_DOCK_ITEM_BEH_CANT_ICONIFY);
	gtk_container_add (GTK_CONTAINER (search_item), obj->priv->search);
	gdl_dock_object_bind (GDL_DOCK_OBJECT (search_item),
			      G_OBJECT (GDL_DOCK_OBJECT_GET_MASTER (obj)));
#endif

#ifdef BUILD_PLAYLIST
	GtkWidget *playlist_item =
	    gdl_dock_item_new_with_stock ("Playlist", _("Playlists"),
					  GTK_STOCK_INDEX,
					  GDL_DOCK_ITEM_BEH_NEVER_HORIZONTAL
					  |
					  GDL_DOCK_ITEM_BEH_CANT_ICONIFY);
	gtk_container_add (GTK_CONTAINER (playlist_item), obj->priv->playlist);
	gdl_dock_object_bind (GDL_DOCK_OBJECT (playlist_item),
			      G_OBJECT (GDL_DOCK_OBJECT_GET_MASTER (obj)));
#endif

	return GTK_WIDGET (obj);
}

void
bonfire_file_source_show (BonfireFileSource * source)
{
	gtk_action_group_set_sensitive (source->priv->action_group, TRUE);

	if (!source->priv->layout) {
		char *path;

		source->priv->layout = gdl_dock_layout_new (GDL_DOCK (source));
		g_signal_connect (G_OBJECT (source->priv->layout),
				  "notify::dirty",
				  G_CALLBACK (bonfire_file_source_layout_changed_cb),
				  source);

		path = gnome_util_home_file ("bonfire.layout");

		if (g_file_test (path, G_FILE_TEST_EXISTS))
			gdl_dock_layout_load_from_file (source->priv->layout,
							path);

		g_free (path);
	}

	if (!gdl_dock_layout_load_layout (source->priv->layout, "MainWin")) {
		GdlDockItem *chooser_item;
		GdlDockPlaceholder *top_ph, *bottom_ph;

		top_ph =
		    gdl_dock_get_placeholder_by_name (GDL_DOCK (source),
						      "ph_top");
		bottom_ph =
		    gdl_dock_get_placeholder_by_name (GDL_DOCK (source),
						      "ph_bot");

#ifdef BUILD_PREVIEW
		GdlDockItem *viewer_item =
		    gdl_dock_get_item_by_name (GDL_DOCK (source),
					       "Viewer");
		gtk_container_add (GTK_CONTAINER (bottom_ph),
				   GTK_WIDGET (viewer_item));
		gtk_widget_show_all (GTK_WIDGET (viewer_item));
#endif

		chooser_item =
		    gdl_dock_get_item_by_name (GDL_DOCK (source),
					       "Chooser");
		gtk_container_add (GTK_CONTAINER (top_ph),
				   GTK_WIDGET (chooser_item));
		gtk_widget_show_all (GTK_WIDGET (chooser_item));

#ifdef BUILD_SEARCH
		GdlDockItem *search_item =
		    gdl_dock_get_item_by_name (GDL_DOCK (source),
					       "Search");
		gtk_container_add (GTK_CONTAINER (top_ph),
				   GTK_WIDGET (search_item));
		gtk_widget_show_all (GTK_WIDGET (search_item));
#endif

#ifdef BUILD_PLAYLIST
		GdlDockItem *playlist_item =
		    gdl_dock_get_item_by_name (GDL_DOCK (source),
					       "Playlist");
		gtk_container_add (GTK_CONTAINER (top_ph),
				   GTK_WIDGET (playlist_item));
		gtk_widget_show_all (GTK_WIDGET (playlist_item));
#endif
	}
}

void
bonfire_file_source_hide (BonfireFileSource * source)
{
	GdlDockItem *chooser_item;

	gtk_action_group_set_sensitive (source->priv->action_group, FALSE);
	gdl_dock_layout_save_layout (source->priv->layout, "MainWin");

#ifdef BUILD_PREVIEW
	GdlDockItem *viewer_item =
	    gdl_dock_get_item_by_name (GDL_DOCK (source), "Viewer");
	gdl_dock_item_hide_item (GDL_DOCK_ITEM (viewer_item));
#endif

	chooser_item =
	    gdl_dock_get_item_by_name (GDL_DOCK (source), "Chooser");
	gdl_dock_item_hide_item (GDL_DOCK_ITEM (chooser_item));

#ifdef BUILD_SEARCH
	GdlDockItem *search_item =
	    gdl_dock_get_item_by_name (GDL_DOCK (source), "Search");
	gdl_dock_item_hide_item (GDL_DOCK_ITEM (search_item));
#endif

#ifdef BUILD_PLAYLIST
	GdlDockItem *playlist_item = gdl_dock_get_item_by_name (GDL_DOCK (source), "Playlist");
	gdl_dock_item_hide_item (GDL_DOCK_ITEM (playlist_item));
#endif
}

void
bonfire_file_source_add_menu (BonfireFileSource * source,
			      GtkUIManager * ui_manager)
{
	GError *error = NULL;

	gtk_ui_manager_insert_action_group (ui_manager,
					    source->priv->action_group, 0);

	if (!gtk_ui_manager_add_ui_from_string (ui_manager, description, -1, &error)) {
		g_message ("building menus failed: %s", error->message);
		g_error_free (error);
		exit (EXIT_FAILURE);
	}
}

char **
bonfire_file_source_get_uris (BonfireFileSource * source)
{
	char **uris = NULL;
	GSList *list, *iter;
	int i;

	switch (source->priv->focus) {
	case BONFIRE_FILE_SOURCE_CHOOSER_PAGE:
		list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (source->priv->chooser));

		if (list == NULL)
			return NULL;

		list = g_slist_reverse (list);
		uris = g_new0 (char *, g_slist_length (list) + 1);
		for (iter = list, i = 0; iter; i++, iter = iter->next)
			uris[i] = (char *) iter->data;
		g_slist_free (list);
		break;
#ifdef BUILD_SEARCH
	case BONFIRE_FILE_SOURCE_SEARCH_PAGE:
		uris =
		    bonfire_search_get_selected_uris (BONFIRE_SEARCH
						      (source->priv->
						       search));
		break;
#endif
#ifdef BUILD_PLAYLIST
	case BONFIRE_FILE_SOURCE_PLAYLIST_PAGE:
		uris =
		    bonfire_playlist_get_selected_uris (BONFIRE_PLAYLIST
							(source->priv->
							 playlist));
		break;
#endif
	case BONFIRE_FILE_SOURCE_NO_PAGE:
		uris = NULL;
		break;
	}
 	return uris;
}

static void
bonfire_file_source_display_toggled_cb (GtkToggleAction * action,
					BonfireFileSource * source)
{
	char *name;

	g_object_get (G_OBJECT (action), "name", &name, NULL);
	if (name) {
		GdlDockItem *item;

		item = gdl_dock_get_item_by_name (GDL_DOCK (source), name);
		g_free (name);
		if (!item)
			return;

		if (gtk_toggle_action_get_active (action) == TRUE)
			gdl_dock_item_show_item (item);
		else
			gdl_dock_item_hide_item (item);
	}
}

static void
bonfire_file_source_foreach_item (GdlDockObject * item,
				  BonfireFileSource * source)
{
	GtkToggleAction *action;
	BonfireFileSourcePage page;

	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action (source->priv->action_group,
								 item->name));
	if (!action)
		return;

	g_signal_handlers_block_by_func (action,
					 G_CALLBACK (bonfire_file_source_display_toggled_cb),
					 source);

	if (!strcmp (item->name, "Chooser"))
		page = BONFIRE_FILE_SOURCE_CHOOSER_PAGE;
#ifdef BUILD_PLAYLIST
	else if (!strcmp (item->name, "Playlist"))
		page = BONFIRE_FILE_SOURCE_PLAYLIST_PAGE;
#endif
#ifdef BUILD_SEARCH
	else if (!strcmp (item->name, "Search"))
		page = BONFIRE_FILE_SOURCE_SEARCH_PAGE;
#endif
	else
		page = BONFIRE_FILE_SOURCE_NO_PAGE;

	if (GDL_DOCK_OBJECT_ATTACHED (item)) {
		gtk_toggle_action_set_active (action, TRUE);
		
		if (source->priv->focus == BONFIRE_FILE_SOURCE_NO_PAGE
		&&  GTK_WIDGET_MAPPED (GDL_DOCK_ITEM (item)->child)
		&&  GTK_WIDGET_VISIBLE (GDL_DOCK_ITEM (item)->child)) {
			gtk_widget_grab_focus (GTK_WIDGET (GDL_DOCK_ITEM (item)->child));
			bonfire_file_source_focus_changed (source, page);
		}
	}
	else {
		gtk_toggle_action_set_active (action, FALSE);
		if (source->priv->focus == page)
			bonfire_file_source_focus_changed (source,
							   BONFIRE_FILE_SOURCE_NO_PAGE);
	}

	g_signal_handlers_unblock_by_func (action,
					   G_CALLBACK (bonfire_file_source_display_toggled_cb),
					   source);
}

static void
bonfire_file_source_layout_changed_cb (GObject * object,
				       GParamSpec * pspec,
				       BonfireFileSource * source)
{
	if (!strcmp (pspec->name, "dirty")) {
		gboolean dirty;

		g_object_get (object, "dirty", &dirty, NULL);

		if (dirty) {
			gdl_dock_master_foreach (GDL_DOCK_OBJECT_GET_MASTER (GDL_DOCK_OBJECT (source)),
						 (GFunc) bonfire_file_source_foreach_item,
						 source);
			if (source->priv->focus == BONFIRE_FILE_SOURCE_NO_PAGE) {
				/* try again to see if we can't find a visible pane:
				 * this can happen if a pane with the focus disappear
				 * and it is looked up last in the list */
				gdl_dock_master_foreach (GDL_DOCK_OBJECT_GET_MASTER (GDL_DOCK_OBJECT (source)),
							 (GFunc) bonfire_file_source_foreach_item,
							 source);
			}
		}
	}
}

static void
bonfire_file_source_focus_changed (BonfireFileSource * source,
				   BonfireFileSourcePage page)
{
	BonfireFileSourcePage prev_page;

	g_signal_emit (source,
		       bonfire_file_source_signals [SELECTION_CHANGED_SIGNAL],
		       0, (page == BONFIRE_FILE_SOURCE_NO_PAGE));

	if (source->priv->focus == page)
		return;

	prev_page = source->priv->focus;
	source->priv->focus = page;

	switch (prev_page) {
	case BONFIRE_FILE_SOURCE_CHOOSER_PAGE:
		g_signal_handlers_block_by_func (source->priv->chooser,
						 bonfire_file_source_chooser_selection_changed_cb,
						 source);
		gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER (source->priv->chooser));
		g_signal_handlers_unblock_by_func (source->priv->chooser,
						   bonfire_file_source_chooser_selection_changed_cb,
						   source);
		break;
#ifdef BUILD_SEARCH
	case BONFIRE_FILE_SOURCE_SEARCH_PAGE:
		g_signal_handlers_block_by_func (source->priv->search,
						 bonfire_file_source_search_selection_changed_cb,
						 source);
		bonfire_search_unselect_all (BONFIRE_SEARCH (source->priv->search));
		g_signal_handlers_unblock_by_func (source->priv->search,
						   bonfire_file_source_search_selection_changed_cb,
						   source);
		break;
#endif
#ifdef BUILD_PLAYLIST
	case BONFIRE_FILE_SOURCE_PLAYLIST_PAGE:
		g_signal_handlers_block_by_func (source->priv->playlist,
						 bonfire_file_source_playlist_selection_changed_cb,
						 source);
		bonfire_playlist_unselect_all (BONFIRE_PLAYLIST (source->priv->playlist));
		g_signal_handlers_unblock_by_func (source->priv->playlist,
						   bonfire_file_source_playlist_selection_changed_cb,
						   source);
		break;
#endif
	case BONFIRE_FILE_SOURCE_NO_PAGE:
		break;
	}

	g_signal_emit (source,
		       bonfire_file_source_signals [SELECTION_CHANGED_SIGNAL],
		       0, (page == BONFIRE_FILE_SOURCE_NO_PAGE));
}

static void
bonfire_file_source_selection_changed (BonfireFileSource *source)
{

#ifdef BUILD_PREVIEW
	char **uris = bonfire_file_source_get_uris (source);
	if (uris) {
		const char *uri;

		uri = *uris;
		bonfire_player_set_uri (BONFIRE_PLAYER (source->priv->viewer), uri);
		g_strfreev (uris);
	}
	else
		bonfire_player_set_uri (BONFIRE_PLAYER (source->priv->viewer),
					NULL);
#endif
}

static void
bonfire_file_source_chooser_activated_cb (GtkFileChooser * chooser,
					  BonfireFileSource * source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_CHOOSER_PAGE);

	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_chooser_selection_changed_cb (GtkFileChooser * chooser,
						  BonfireFileSource *
						  source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_CHOOSER_PAGE);

	bonfire_file_source_selection_changed (source);
}

#ifdef BUILD_SEARCH
static void
bonfire_file_source_search_activated_cb (BonfireSearch * search,
					 BonfireFileSource * source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_SEARCH_PAGE);

	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_search_selection_changed_cb (BonfireSearch * search,
						 BonfireFileSource *
						 source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_SEARCH_PAGE);

	bonfire_file_source_selection_changed (source);
}
#endif

#ifdef BUILD_PLAYLIST
static void
bonfire_file_source_playlist_activated_cb (BonfirePlaylist * playlist,
					   BonfireFileSource * source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_PLAYLIST_PAGE);

	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_playlist_selection_changed_cb (BonfirePlaylist *playlist,
						   BonfireFileSource *source)
{
	bonfire_file_source_focus_changed (source,
					   BONFIRE_FILE_SOURCE_PLAYLIST_PAGE);

	bonfire_file_source_selection_changed (source);
}
#endif

#endif				/* BUILD_GDL */
