/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/***************************************************************************
 *            main.c
 *
 *  Sat Jun 11 12:00:29 2005
 *  Copyright  2005  Philippe Rouquier	
 *  <bonfire-app@wanadoo.fr>
 ****************************************************************************/

#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <locale.h>

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>

#include <gst/gst.h>

#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include <nautilus-burn-recorder.h>

#include <gconf/gconf-client.h>

#ifdef HAVE_LIBNOTIFY
#include <libnotify/notify.h>
#endif

#include "menu.h"
#include "project.h"
#include "blank-dialog.h"
#include "disc-copy-dialog.h"
#include "disc.h"

#ifdef BUILD_GDL
#include <gdl/gdl-dock-layout.h>
#include <gdl/gdl-dock-bar.h>
#include <gdl/gdl-dock.h>
#include <gdl/gdl-dock-item.h>

#include "file-src-gdl.h"
#else
#include "file-src.h"
#endif

static GConfClient *client;
gchar *project_name;
gchar *iso_uri;
gint audio_project;
gint data_project;
gint copy_project;
gint debug;

static const GOptionEntry options [] = {
	{ "project", 'p', G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING, &project_name,
	  N_("Open the specified project"),
	  N_("PROJECT") },

	{ "audio", 'a', 0, G_OPTION_ARG_NONE, &audio_project,
	  N_("Open an empty audio project"),
	  NULL },

	{ "data", 'd', 0, G_OPTION_ARG_NONE, &data_project,
          N_("Open an empty data project"),
          NULL },

	{ "copy", 'c', 0, G_OPTION_ARG_NONE, &copy_project,
	  N_("Copy a disc"),
	  NULL },

	{ "iso", 'i', G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING, &iso_uri,
          N_("Uri of an image file or a cue file to be burnt (autodetected)"),
          NULL },

	{ "debug", 'g', 0, G_OPTION_ARG_NONE, &debug,
	  N_("Display debug statements on stdout"),
	  NULL },

	{ NULL }
};

static void
on_selection_changed (BonfireProject *project, gboolean empty, BonfireApp *app)
{
	GtkAction *action;

	app->empty = empty;
	if (empty) { /* the project / file selection is not empty */
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Delete");
		gtk_action_set_sensitive (action, TRUE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/DeleteAll");
		gtk_action_set_sensitive (action, TRUE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/CDMenu/Burn");
		gtk_action_set_sensitive (action, TRUE);

		if (app->saved
		&& !app->loading) {
			app->saved = FALSE;
			action = gtk_ui_manager_get_action (app->manager,
							    "/menubar/ProjectMenu/Save");
			gtk_action_set_sensitive (action, TRUE);
		}
	}
	else {
		if (!app->saved) {
			app->saved = TRUE;
			action = gtk_ui_manager_get_action (app->manager,
							    "/menubar/ProjectMenu/Save");
			gtk_action_set_sensitive (action, FALSE);
		}

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Delete");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/DeleteAll");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/CDMenu/Burn");
		gtk_action_set_sensitive (action, FALSE);
	}

	if (app->loading) {
		char **uris;

		app->loading = FALSE;
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/New");
		gtk_action_set_sensitive (action, TRUE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Open");
		gtk_action_set_sensitive (action, TRUE);

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Add");
		uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
		if (uris) {
			g_strfreev (uris);
			gtk_action_set_sensitive (action, TRUE);
		}
		else
			gtk_action_set_sensitive (action, FALSE);
	}
}

static gboolean
on_delete_cb (GtkWidget *window, GdkEvent *event, BonfireApp *app)
{
	return FALSE;
}

static gboolean
on_destroy_cb (GtkWidget *window, GdkEvent *event, BonfireApp *app)
{
	gtk_main_quit ();

	return FALSE;
}

gboolean
bonfire_unsaved_project (BonfireApp *app)
{
	gint result;
	GtkWidget *message;

	/* ask confirmation */
	message = gtk_message_dialog_new (GTK_WINDOW (app->mainwin),
					  GTK_DIALOG_DESTROY_WITH_PARENT |
					  GTK_DIALOG_MODAL,
					  GTK_MESSAGE_QUESTION,
					  GTK_BUTTONS_NONE,
					  _("Your current project is not saved:"));

	gtk_window_set_title (GTK_WINDOW (message), _("Unsaved project"));

	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
						  _("Do you want to save it?"));

	gtk_dialog_add_buttons (GTK_DIALOG (message),
			        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			        _("save project"), GTK_RESPONSE_YES,
			        _("don't save project"), GTK_RESPONSE_NO,
			        NULL);

	result = gtk_dialog_run (GTK_DIALOG (message));
	gtk_widget_destroy (message);

	if (result == GTK_RESPONSE_YES)
		return bonfire_project_save_project (BONFIRE_PROJECT (app->project));

	if (result != GTK_RESPONSE_NO)
		return FALSE;
	
	return TRUE;
}

void
on_project_chosen_cb (GtkNotebook *notebook,
		      GtkNotebookPage *page,
		      gint num,
		      BonfireApp *app)
{
	GtkAction *action;
	char **uris;

#ifdef BUILD_GDL

	GdlDockItem *item;
	char *stock_icon;

	item = gdl_dock_get_item_by_name (GDL_DOCK (app->source), "CD");
	g_object_set (G_OBJECT (item), "long_name", _("Disc contents"),
		      NULL);
	gdl_dock_object_detach (GDL_DOCK_OBJECT (item), TRUE);
	bonfire_file_source_show (BONFIRE_FILE_SOURCE (app->source));

	gtk_widget_show_all (app->source);

	if (!GDL_DOCK_OBJECT_ATTACHED (GDL_DOCK_OBJECT (item))) {
		GdlDockPlaceholder *placeh;

		placeh = gdl_dock_get_placeholder_by_name (GDL_DOCK
							   (app->source),
							   "ph_right");
		gtk_container_add (GTK_CONTAINER (placeh),
				   GTK_WIDGET (item));
	}

	stock_icon = bonfire_project_get_content_icon (BONFIRE_PROJECT (app->project));
	g_object_set (G_OBJECT (item), "stock_id", stock_icon, NULL);
	g_free (stock_icon);

#else

	bonfire_file_source_show (BONFIRE_FILE_SOURCE (app->source));

#endif

	g_signal_handlers_disconnect_by_func (G_OBJECT (notebook),
					      on_project_chosen_cb, app);


	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/EditMenu/Add");

	uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
	if (uris) {
		g_strfreev (uris);
		gtk_action_set_sensitive (action, TRUE);
	}
	else
		gtk_action_set_sensitive (action, FALSE);

	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/ProjectMenu/New");
	gtk_action_set_sensitive (action, TRUE);
}

static void
_new_project (BonfireApp *app)
{
	GtkAction *action;

#ifdef BUILD_GDL

	GdlDockItem *item;

	item = gdl_dock_get_item_by_name (GDL_DOCK (app->source), "CD");
	g_object_set (G_OBJECT (item), "long_name", "", NULL);

#endif
	/* FIXME: if a project is opened in between above function will not
	 * be called */

	bonfire_project_new_project (BONFIRE_PROJECT (app->project));
	bonfire_file_source_hide (BONFIRE_FILE_SOURCE (app->source));

	app->saved = TRUE;
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/ProjectMenu/New");
	gtk_action_set_sensitive (action, FALSE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/ProjectMenu/Open");
	gtk_action_set_sensitive (action, TRUE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/ProjectMenu/Save");
	gtk_action_set_sensitive (action, FALSE);
	action =
	    gtk_ui_manager_get_action (app->manager,
				       "/menubar/EditMenu/Add");
	gtk_action_set_sensitive (action, FALSE);
	action =
	    gtk_ui_manager_get_action (app->manager,
				       "/menubar/EditMenu/Delete");
	gtk_action_set_sensitive (action, FALSE);
	action =
	    gtk_ui_manager_get_action (app->manager,
				       "/menubar/EditMenu/DeleteAll");
	gtk_action_set_sensitive (action, FALSE);
	action =
	    gtk_ui_manager_get_action (app->manager,
				       "/menubar/CDMenu/Burn");
	gtk_action_set_sensitive (action, FALSE);
	g_signal_connect (G_OBJECT (app->project), "switch-page",
			  G_CALLBACK (on_project_chosen_cb), app);
}

void
on_new_cd_cb (GtkAction *action, BonfireApp *app)
{
	_new_project (app);
}

void
on_new_audio_cd_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_new_audio_project (BONFIRE_PROJECT (app->project));
}

void
on_new_data_cd_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_new_data_project (BONFIRE_PROJECT (app->project));
}

void
on_new_copy_cd_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_new_copy_disc (BONFIRE_PROJECT (app->project));
}

void
on_new_iso_cd_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_new_iso (BONFIRE_PROJECT (app->project), NULL);
}

static void
_load_project (BonfireApp *app, const char *project)
{
	GtkAction *action;
	BonfireDiscResult result;

	result = bonfire_project_open_project (BONFIRE_PROJECT (app->project),
					       project);
	if (result == BONFIRE_DISC_LOADING) {
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/New");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Open");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Save");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Add");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Delete");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/DeleteAll");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/CDMenu/Burn");
		gtk_action_set_sensitive (action, FALSE);

		app->saved = TRUE;
		app->loading = TRUE;
	}
	else if (result == BONFIRE_DISC_OK) {
		char **uris;

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Add");
		uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
		if (uris) {
			g_strfreev (uris);
			gtk_action_set_sensitive (action, TRUE);
		}
		else
			gtk_action_set_sensitive (action, FALSE);

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Save");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Delete");
		gtk_action_set_sensitive (action, TRUE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/DeleteAll");
		gtk_action_set_sensitive (action, TRUE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/CDMenu/Burn");
		gtk_action_set_sensitive (action, TRUE);

		app->saved = TRUE;
	}
	else if (result == BONFIRE_DISC_ERROR_EMPTY_SELECTION) {
		char **uris;

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Add");
		uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
		if (uris) {
			g_strfreev (uris);
			gtk_action_set_sensitive (action, TRUE);
		}
		else
			gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Save");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/Delete");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/EditMenu/DeleteAll");
		gtk_action_set_sensitive (action, FALSE);
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/CDMenu/Burn");
		gtk_action_set_sensitive (action, FALSE);

		app->saved = TRUE;
	}
	else if (result != BONFIRE_DISC_CANCELLED) {
		_new_project (app);
	}
}

void
on_open_cd_cb (GtkAction *action, BonfireApp *app)
{

	if (app->empty
	&&  !app->saved
	&&  !bonfire_unsaved_project (app))
		return;

	_load_project (app, NULL);
}

void
on_save_cd_cb (GtkAction *action, BonfireApp *app)
{
	app->saved = bonfire_project_save_project (BONFIRE_PROJECT (app->project));
	if (app->saved) {
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Save");
		gtk_action_set_sensitive (action, FALSE);
	}
}

void
on_save_as_cd_cb (GtkAction *action, BonfireApp *app)
{
	gboolean state;

	state = bonfire_project_save_project_as (BONFIRE_PROJECT (app->project));
	if (state && !app->saved) {
		app->saved = TRUE;
		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/Save");
		gtk_action_set_sensitive (action, FALSE);
	}
}

void
on_exit_cb (GtkAction *action, BonfireApp *app)
{
	gtk_widget_destroy (app->mainwin);
}

void
on_erase_cb (GtkAction *action, BonfireApp *app)
{
	GtkWidget *dialog;
	GtkWidget *toplevel;

	dialog = bonfire_blank_dialog_new ();
	toplevel = gtk_widget_get_toplevel (app->mainwin);

	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);

	gtk_widget_show_all (dialog);
}

void
on_burn_cb (GtkAction *button, BonfireApp *app)
{
	bonfire_project_burn_contents (BONFIRE_PROJECT (app->project));
}

void
on_disc_info_cb (GtkAction *button, BonfireApp *app)
{

}

void
on_add_cb (GtkAction *action, BonfireApp *app)
{
	char **uris;

	uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
	bonfire_project_add_uris (BONFIRE_PROJECT (app->project), uris);
	if (uris)
		g_strfreev (uris);
}

void
on_remove_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_remove_selected_uris (BONFIRE_PROJECT (app->project));
}

void
on_remove_all_cb (GtkAction *action, BonfireApp *app)
{
	bonfire_project_clear (BONFIRE_PROJECT (app->project));
}

void
on_about_cb (GtkAction *action, BonfireApp *app)
{
	GtkWidget *dialog;
	const char *authors[] = { "Philippe Rouquier", NULL };
	GdkPixbuf *logo;

	logo =
	    gdk_pixbuf_new_from_file (BONFIRE_DATADIR
				      "/icon-final-128x128.png", NULL);

	dialog = gtk_about_dialog_new ();
	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (dialog), "Bonfire");
	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog),
					"Copyright (c) Philippe Rouquier");
	gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (dialog), authors);
	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog),
				       "Disc burning tool");
	gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (dialog), logo);
	g_object_unref (logo);
	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), VERSION);
	gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (dialog),
						 _("translator-credits"));

	gtk_window_set_transient_for (GTK_WINDOW (dialog),
				      GTK_WINDOW (app->mainwin));
	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_position (GTK_WINDOW (dialog),
				 GTK_WIN_POS_CENTER_ON_PARENT);

	gtk_widget_show (dialog);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

static void
on_activated_cb (BonfireFileSource *source, BonfireApp *app)
{
	char **uris;

	uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE
					     (app->source));
	if (uris == NULL)
		return;

	bonfire_project_add_uris (BONFIRE_PROJECT (app->project), uris);
	g_strfreev (uris);
}

static void
on_source_changed_cb (BonfireFileSource *source,
		      gboolean empty,
		      BonfireApp *app)
{
	GtkAction *action;
	char **uris;

	if (app->loading)
		return;

	action = gtk_ui_manager_get_action (app->manager, "/menubar/EditMenu/Add");
	uris = bonfire_file_source_get_uris (BONFIRE_FILE_SOURCE (app->source));
	if (uris) {
		g_strfreev (uris);
		gtk_action_set_sensitive (action, TRUE);
	}
	else
		gtk_action_set_sensitive (action, FALSE);
}

BonfireApp *
bonfire_app_create_app (void)
{
	BonfireApp *app;
	GtkWidget *menubar, *toolbar;
	GtkActionGroup *action_group;
	GtkAction *action;
	GtkAccelGroup *accel_group;
	GError *error = NULL;

	/* New window */
	app = g_new0 (BonfireApp, 1);
	app->mainwin = gnome_app_new ("Bonfire", NULL);

	gtk_window_set_icon_name (GTK_WINDOW (app->mainwin),
				  "disc-burner");
	gtk_window_set_icon_from_file (GTK_WINDOW (app->mainwin),
				       BONFIRE_DATADIR G_DIR_SEPARATOR_S "icon-final-48x48.png",
				       NULL);

	g_signal_connect (G_OBJECT (app->mainwin), "delete-event",
			  G_CALLBACK (on_delete_cb), app);
	g_signal_connect (G_OBJECT (app->mainwin), "destroy",
			  G_CALLBACK (on_destroy_cb), app);

	/* status bar FIXME: what for ??? */
	app->statusbar = gtk_statusbar_new ();
	gnome_app_set_statusbar (GNOME_APP (app->mainwin), app->statusbar);

	/* menu and toolbar */
	action_group = gtk_action_group_new ("MenuActions");
	gtk_action_group_set_translation_domain (action_group, NULL);
	gtk_action_group_add_actions (action_group, entries,
				      G_N_ELEMENTS (entries), app);

	app->manager = gtk_ui_manager_new ();
	gtk_ui_manager_insert_action_group (app->manager, action_group, 0);

	if (!gtk_ui_manager_add_ui_from_string
	    (app->manager, description, -1, &error)) {
		g_message ("building menus failed: %s", error->message);
		g_error_free (error);
	}

	menubar = gtk_ui_manager_get_widget (app->manager, "/menubar");
	gnome_app_set_menus (GNOME_APP (app->mainwin),
			     GTK_MENU_BAR (menubar));
	toolbar = gtk_ui_manager_get_widget (app->manager, "/toolbar");
	gnome_app_set_toolbar (GNOME_APP (app->mainwin),
			       GTK_TOOLBAR (toolbar));

	app->saved = TRUE;
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/ProjectMenu/Save");
	gtk_action_set_sensitive (action, FALSE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/EditMenu/Add");
	gtk_action_set_sensitive (action, FALSE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/EditMenu/Delete");
	gtk_action_set_sensitive (action, FALSE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/EditMenu/DeleteAll");
	gtk_action_set_sensitive (action, FALSE);
	action = gtk_ui_manager_get_action (app->manager,
					    "/menubar/CDMenu/Burn");
	gtk_action_set_sensitive (action, FALSE);

	/* Window contents */
	app->project = bonfire_project_new ();
	g_signal_connect (G_OBJECT (app->project), "selection_changed",
			  G_CALLBACK (on_selection_changed), app);

	app->source = bonfire_file_source_new ();
	g_signal_connect (G_OBJECT (app->source), "activated",
			  G_CALLBACK (on_activated_cb), app);
	g_signal_connect (G_OBJECT (app->source), "selection_changed",
			  G_CALLBACK (on_source_changed_cb), app);

#ifdef BUILD_GDL

	GtkWidget *gdlitem;
	GdlDockPlaceholder *placeh;

	gtk_widget_show (app->source);
	gnome_app_set_contents (GNOME_APP (app->mainwin), app->source);

	/* Insert the cd in the source */
	gdlitem = gdl_dock_item_new_with_stock ("CD",
						NULL,//_("Choose a disc type"),
						NULL,//GTK_STOCK_CDROM,
						GDL_DOCK_ITEM_BEH_NEVER_HORIZONTAL |
						GDL_DOCK_ITEM_BEH_CANT_CLOSE |
						GDL_DOCK_ITEM_BEH_CANT_ICONIFY |
						GDL_DOCK_ITEM_BEH_NEVER_FLOATING);


	gtk_container_add (GTK_CONTAINER (gdlitem), app->project);
	gtk_widget_show_all (gdlitem);
	gdl_dock_item_bind (GDL_DOCK_ITEM (gdlitem), app->source);
	bonfire_project_new_project (BONFIRE_PROJECT (app->project));

	placeh =
	    gdl_dock_get_placeholder_by_name (GDL_DOCK (app->source),
					      "ph_right");
	gtk_container_add (GTK_CONTAINER (placeh), GTK_WIDGET (gdlitem));

#else

	GtkWidget *pane;

	pane = gtk_hpaned_new ();
	gtk_paned_pack1 (GTK_PANED (pane), app->project, TRUE, FALSE);
	gtk_paned_pack2 (GTK_PANED (pane), app->source, TRUE, FALSE);

	gtk_widget_show_all (app->project);
	gtk_widget_hide (app->source);

	gnome_app_set_contents (GNOME_APP (app->mainwin), pane);
	gtk_widget_show (pane);

#endif				/* BUILD_GDL */

	bonfire_file_source_add_menu (BONFIRE_FILE_SOURCE (app->source),
				      app->manager);

	/* add accelerators now */
	accel_group = gtk_ui_manager_get_accel_group (app->manager);
	gtk_window_add_accel_group (GTK_WINDOW (app->mainwin),
				    accel_group);

	gtk_window_set_default_size (GTK_WINDOW (app->mainwin), 800, 640);
	return app;
}

int
main (int argc, char **argv)
{
	BonfireApp *app;
	GnomeProgram *program;
	GOptionContext *context;

	context = g_option_context_new (NULL);
	g_option_context_add_main_entries (context,
					   options,
					   GETTEXT_PACKAGE);

	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
				      argc, argv,
				      GNOME_PARAM_GOPTION_CONTEXT, context,
				      GNOME_PARAM_HUMAN_READABLE_NAME, _("CD/DVD burning"),
				      NULL);

	bonfire_utils_register_icons ();

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	if (!g_thread_supported ())
		g_thread_init (NULL);

	gnome_vfs_init ();
	gst_init (&argc, &argv);

#ifdef HAVE_LIBNOTIFY
	notify_init ("Bonfire");
#endif

	client = gconf_client_get_default ();

	app = bonfire_app_create_app ();
	if (app == NULL)
		return 1;

	if (audio_project) {
	     	g_signal_connect (G_OBJECT (app->project), "switch-page",
			 	  G_CALLBACK (on_project_chosen_cb), app);
		bonfire_project_new_audio_project (BONFIRE_PROJECT (app->project));
	}
	else if (data_project) {
    	 	g_signal_connect (G_OBJECT (app->project), "switch-page",
				  G_CALLBACK (on_project_chosen_cb), app);
		bonfire_project_new_data_project (BONFIRE_PROJECT (app->project));
	}
	else if (copy_project) {
     		g_signal_connect (G_OBJECT (app->project), "switch-page",
				  G_CALLBACK (on_project_chosen_cb), app);

		bonfire_project_new_copy_disc (BONFIRE_PROJECT (app->project));
	}
	else if (iso_uri) {
     		g_signal_connect (G_OBJECT (app->project), "switch-page",
				  G_CALLBACK (on_project_chosen_cb), app);

		bonfire_project_new_iso (BONFIRE_PROJECT (app->project), iso_uri);
	}
	else if (project_name) {
     		g_signal_connect (G_OBJECT (app->project), "switch-page",
				  G_CALLBACK (on_project_chosen_cb), app);

		_load_project (app, project_name);
	}
	else {
		GtkAction *action;

		action = gtk_ui_manager_get_action (app->manager,
						    "/menubar/ProjectMenu/New");
		gtk_action_set_sensitive (action, FALSE);

		bonfire_project_new_project (BONFIRE_PROJECT (app->project));
     		g_signal_connect (G_OBJECT (app->project), "switch-page",
				  G_CALLBACK (on_project_chosen_cb), app);
	}

	gtk_widget_show (app->mainwin);
	gtk_main ();

	g_object_unref (program);
	g_free (app);
	gst_deinit ();

	g_object_unref (client);
	client = NULL;

	return 0;
}
