/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *            menu.h
 *
 *  Sat Jun 11 12:00:29 2005
 *  Copyright  2005  Philippe Rouquier	
 *  <bonfire-app@wanadoo.fr>
 ****************************************************************************/

#ifndef _MENU_H
#define _MENU_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtkstock.h>
#include <gtk/gtkaction.h>
#include <gtk/gtktoggleaction.h>

#include "utils.h"

	struct _BonfireApp {
		GtkWidget *mainwin;
		GtkWidget *source;
		GtkWidget *project;
		GtkWidget *statusbar;
		GtkUIManager *manager;
		gboolean empty;
		gboolean saved;
		gboolean loading;
	};
	typedef struct _BonfireApp BonfireApp;


	void on_new_cd_cb (GtkAction *action, BonfireApp *app);
	void on_new_audio_cd_cb (GtkAction *action, BonfireApp *app);
	void on_new_data_cd_cb (GtkAction *action, BonfireApp *app);
	void on_new_copy_cd_cb (GtkAction *action, BonfireApp *app);
	void on_new_iso_cd_cb (GtkAction *action, BonfireApp *app);

	void on_open_cd_cb (GtkAction *action, BonfireApp *app);
	void on_save_cd_cb (GtkAction *action, BonfireApp *app);
	void on_save_as_cd_cb (GtkAction *action, BonfireApp *app);
	void on_erase_cb (GtkAction *action, BonfireApp *app);
	void on_exit_cb (GtkAction *action, BonfireApp *app);

	void on_add_cb (GtkAction *action, BonfireApp *app);
	void on_remove_cb (GtkAction *action, BonfireApp *app);
	void on_remove_all_cb (GtkAction *action, BonfireApp *app);
	void on_burn_cb (GtkAction *action, BonfireApp *app);
	void on_disc_info_cb (GtkAction *action, BonfireApp *app);
	void on_about_cb (GtkAction *action, BonfireApp *app);

	static GtkActionEntry entries[] = {
		{"ProjectMenu", NULL, N_("Project")},
		{"ViewMenu", NULL, N_("View")},
		{"EditMenu", NULL, N_("Edit")},
		{"CDMenu", NULL, N_("Recorder")},
		{"HelpMenu", NULL, N_("Help")},

		{"New", GTK_STOCK_NEW, N_("New project"), NULL,
		 N_("Create a new project"), NULL},
		{"NewChoose", GTK_STOCK_NEW, N_("Empty project"), NULL,
		 N_("Let you choose your new project"), G_CALLBACK (on_new_cd_cb)},
		{"NewAudio", NULL, N_("New audio project"), NULL,
		 N_("Create a new audio project"), G_CALLBACK (on_new_audio_cd_cb)},
		{"NewData", NULL, N_("New data project"), NULL,
		 N_("Create a new data project"), G_CALLBACK (on_new_data_cd_cb)},
		{"NewCopy", NULL, N_("Copy disc"), NULL,
		 N_("Copy a disc"), G_CALLBACK (on_new_copy_cd_cb)},
		{"NewIso", NULL, N_("Burn an image"), NULL,
		 N_("Burn an image"), G_CALLBACK (on_new_iso_cd_cb)},

		{"Open", GTK_STOCK_OPEN, N_("_Open"), NULL,
		 N_("Open a project"), G_CALLBACK (on_open_cd_cb)},
		{"Save", GTK_STOCK_SAVE, N_("_Save"), NULL,
		 N_("Save current project"), G_CALLBACK (on_save_cd_cb)},
		{"SaveAs", GTK_STOCK_SAVE_AS, N_("_Save as"), NULL,
		 N_("Save current project to a different location"), G_CALLBACK (on_save_as_cd_cb)},
		{"Exit", GTK_STOCK_QUIT, N_("E_xit"), NULL,
		 N_("Exit the program"), G_CALLBACK (on_exit_cb)},
		{"Erase", GTK_STOCK_CLEAR, N_("_Erase"), NULL,
		 N_("Erase a disc"), G_CALLBACK (on_erase_cb)},
		{"Burn", GTK_STOCK_CDROM, N_("_Burn"), NULL,
		 N_("Burn a disc"), G_CALLBACK (on_burn_cb)},
		{"DiscInfo", GTK_STOCK_CDROM, N_("_Disc Info"), NULL,
		 N_("Display information on blank discs currently inserted"),
		 G_CALLBACK (on_disc_info_cb)},
		{"Add", GTK_STOCK_ADD, N_("_Add"), NULL,
		 N_("Add files to the project"), G_CALLBACK (on_add_cb)},
		{"Delete", GTK_STOCK_REMOVE, N_("Dele_te"), NULL,
		 N_("Delete the selected files from the project"),
		 G_CALLBACK (on_remove_cb)},
		{"DeleteAll", GTK_STOCK_DELETE, N_("De_lete all"), NULL,
		 N_("Delete all files from the project"),
		 G_CALLBACK (on_remove_all_cb)},
		{"About", GTK_STOCK_ABOUT, N_("About"), NULL, N_("About"),
		 G_CALLBACK (on_about_cb)}
	};


	static const char *description = {
		"<ui>"
		    "<menubar name='menubar' >"
		    "<menu action='ProjectMenu'>"
		    "<menu action='New'>"
			"<menuitem action='NewAudio'/>"
			"<menuitem action='NewData'/>"
			"<menuitem action='NewCopy'/>"	
			"<menuitem action='NewIso'/>"	
		    "</menu>"
		    "<menuitem action='Open'/>"
		    "<menuitem action='Save'/>"
		    "<menuitem action='SaveAs'/>"
		    "<separator/>"
		    "<menuitem action='Exit'/>"
		    "</menu>"
		    "<menu action='EditMenu'>"
		    "<menuitem action='Add'/>"
		    "<menuitem action='Delete'/>"
		    "<menuitem action='DeleteAll'/>"
		    "</menu>"
		    "<menu action='ViewMenu'>"
		    "<placeholder name='ViewPlaceholder'/>"
		    "</menu>"
		    "<menu action='CDMenu'>"
		    "<menuitem action='Burn'/>"
		    "<menuitem action='Erase'/>"
		    //"<menuitem action='DiscInfo'/>" 
		    "</menu>"
		    "<menu action='HelpMenu'>"
		    "<menuitem action='About'/>"
		    "</menu>"
		    "</menubar>"
		    "<toolbar name='toolbar'>"
		    "<separator/>"
		    "<toolitem action='NewChoose'/>"
		    "<toolitem action='Open'/>"
		    "<toolitem action='Save'/>"
		    "<separator/>"
		    "<toolitem action='Add'/>"
		    "<toolitem action='Delete'/>"
		    "<toolitem action='DeleteAll'/>"
		    "<separator/>"
		    "<toolitem action='Burn'/>"
		    "<toolitem action='Erase'/>"
		    //"<toolitem action='DiscInfo'/>"
		"<separator/>" "</toolbar>" "</ui>"
	};


#ifdef __cplusplus
}
#endif

#endif				/* _MENU_H */
